/*
Variable width font drawing library for DS (and GBA)

Copyright 2007 Damian Yerrick <pinoandchester@pineight.com>

This work is provided 'as-is', without any express or implied
warranty.  In no event will the authors be held liable for any
damages arising from the use of this work.

Permission is granted to anyone to use this work for any purpose,
including commercial applications, and to alter it and redistribute
it freely, subject to the following restrictions:

1. The origin of this work must not be misrepresented; you must
   not claim that you wrote the original work. If you use
   this work in a product, an acknowledgment in the product
   documentation would be appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original work.
3. This notice may not be removed or altered from any source
   distribution.

"Source" is the preferred form of a work for making changes to it.

*/

#ifndef FONTDRAW_H
#define FONTDRAW_H
#include <nds.h>
#include <sys/types.h>

#ifndef BG_OFFSET_SUB
#define BG_OFFSET_SUB ((bg_scroll *)(0x04001010))
#endif

unsigned int fontdraw_charWidth(int glyph);
unsigned int fontdraw_strWidth(const char *s);

/**
 * Returns the number of characters in s that fit within targetWidth pixels.
 */
size_t fontdraw_cutStr(const char *s, int targetWidth);

void fontdraw_setupVRAM(int sub);

// New API

typedef struct VWFWindow {
  u8 left;    // in 8 pixel units on nametable
  u8 top;     // in 8 pixel units on nametable
  u8 width;   // in 8 pixel units on nametable
  u8 height;  // in 8 pixel units on nametable
  u32 *chrBase;
  u8 map;  // in 2 KiB units on VRAM
  u8 core;    // 0: main; 1: sub
  u16 mapTileBase;
} VWFWindow;

void vwfWinClear(const VWFWindow *src);
void vwfWinInit(const VWFWindow *src);
unsigned int vwfPutc(const VWFWindow *w,
             int c,
             int x, int y);
unsigned int vwfPuts(const VWFWindow *src,
                     const char *str,
                     int x, int y);
void vwfRectfill(const VWFWindow *v,
                 int l, int t, int r, int b,
                 int c);
void vwfHline(const VWFWindow *v, int l, int t, int r, int c);
void vwfVline(const VWFWindow *v, int l, int t, int b, int c);
void vwfRect(const VWFWindow *v, int l, int t, int r, int b, int c);

extern const VWFWindow vwfTop, vwfTouch;


#endif
