/*
Simple function for reading both the ARM9 and ARM7 joypads at once
on the DS.

By Damian Yerrick. No rights reserved and ABSOLUTELY NO WARRANTY.
*/

#ifndef XKEYS_H
#define XKEYS_H

#include <nds.h>
#define XKEY_X 0x00010000
#define XKEY_Y 0x00020000
#define XKEY_TOUCH 0x00400000
#define XKEY_OPEN 0x00800000

/**
 * Reads the ARM9 part of the DS gamepad and the last values that
 * the ARM7 touch code read from the ARM7 part of the DS gamepad.
 * The returned bitfield may contain
 * KEY_A, KEY_B, KEY_SELECT, KEY_START, KEY_UP, KEY_DOWN,
 * KEY_LEFT, KEY_RIGHT, KEY_R, KEY_L: GBA keys
 * XKEY_X: X button is being held
 * XKEY_Y: Y button is being held
 * XKEY_TOUCH: Touch screen is being touched (and has valid coords)
 * XKEY_OPEN: Lid is open
 */
static inline u32 readPad(void) {
  return (~(REG_KEYINPUT | (IPC->buttons << 16)))
          & 0x00C303FF;
}

#endif
