/* background and sprite cel packer for GBA/DS


 */

/*
1. Read a raster image in 16 colors.
2. Gamma-correct and write the palette immediately.
3. For each tile read, check if it matches any other tile.  Write
*/

#include <allegro.h>
#include <stdio.h>
#include <math.h>  /* for sqrt() */
#include <string.h>  /* for strerror() */

#define GBA_GAMMA 0  // 1: GBA; 0: DS


/* fputu16() ***************************
   Write a 16-bit integer in little-endian format to a file.
*/
void fputu16(unsigned int n, FILE *fp)
{
  fputc(n & 0xff, fp);
  fputc(n >> 8, fp);
}


#if 0

unsigned char this_tile[600][8][8];
/* I'm not really using is_match() or find_match() this time because
   they didn't help much. */
unsigned int is_match(unsigned int i, unsigned int n,
		      unsigned int x_xor, unsigned int y_xor)
{
  unsigned int x, y;

  for(y = 0; y < 8; y++)
    for(x = 0; x < 8; x++)
      if(this_tile[i][y ^ y_xor][x ^ x_xor] != this_tile[n][y][x])
	return 0;  /* bail on the first mismatch */
  return 1;
}


/* find_match() ************************
   Find the first match for tile n.  If there is no match, return n.
*/
unsigned int find_match(unsigned int n)
{
  unsigned int i = 0;

  if(n == 0)
    return 0;

  for(i = 0; i < n; i++)
    {
      if(is_match(i, n, 0, 0))  /* straight matches */
	return i;
      else if(is_match(i, n, 7, 0))  /* h-flipped matches */
	return i | 0x0200;
      else if(is_match(i, n, 0, 7))  /* v-flipped matches */
	return i | 0x0400;
      else if(is_match(i, n, 7, 7))  /* upside-down matches */
	return i | 0x0600;
    }
  return n;  /* this is a new tile */
}
#endif

static
unsigned int correct_gba_component(unsigned int c) {
#if GBA_GAMMA
  return ((int)sqrt(c * 64) + c) >> 2;
#else
  return c >> 1;
#endif
}


/* rgb2gba() ******************************
   Convert an RGB color in (0..63) to a gamma-corrected GBA color in (0..31).
*/
unsigned int rgb2gba(const RGB *color) {
  return (correct_gba_component(color->b) << 10)
         | (correct_gba_component(color->g) << 5)
         | (correct_gba_component(color->r));
}


int main(int argc, char **argv) {
  BITMAP *bmp;
  PALETTE pal;
  FILE *fout;
  int xx, yy;

  install_allegro(SYSTEM_NONE, &errno, atexit);
  set_color_depth(8);

  if(argc != 3)
    {
      allegro_message("usage: bmp2bg foo.bmp\n");
      return 1;
    }

  bmp = load_bitmap(argv[1], pal);
  if (!bmp) {
    allegro_message("Could not load %s: %s\n",
  	      argv[1], strerror(errno));
    return 1;
  }

  fout = fopen(argv[2], "wb");
  if (!fout) {
    destroy_bitmap(bmp);
    allegro_message("Could not open %s for writing: %s\n",
                    argv[2], strerror(errno));
    return 1;
  }

  /* Write palette */
  for (yy = 0; yy < 16; yy++) {
    fputu16(rgb2gba(&(pal[yy])), fout);
  }

  /* Write pattable */
  for (yy = 0; yy < bmp->h; yy += 8) {
    for (xx = 0; xx < bmp->w; xx += 8) {
      unsigned int x, y;

       for (y = 0; y < 8; y++) {
         for (x = 0; x < 8; x += 2) {
          unsigned int b = getpixel(bmp, x + xx, y + yy) & 0x0f;
          b |= getpixel(bmp, x + xx + 1, y + yy) << 4;
          fputc(b, fout);
        }
      }
    }
  }
  fclose(fout);
  destroy_bitmap(bmp);
  return 0;
}
END_OF_MAIN();
